@@ -4,7 +4,7 @@ package WebService::Avalara::AvaTax;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.009';     # TRIAL VERSION
+our $VERSION = '0.017';     # TRIAL VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -29,6 +29,8 @@ use utf8;
 
 use Carp;
 use Const::Fast;
+use DateTime;
+use DateTime::Format::XSD;
 use English '-no_match_vars';
 use Moo;
 use Package::Stash;
@@ -93,27 +95,6 @@ sub _new_service {
     );
 }
 
-#pod =head1 SEE ALSO
-#pod
-#pod =over
-#pod
-#pod =item L<Avalara Developer Network|http://developer.avalara.com/>
-#pod
-#pod Official source for Avalara developer information, including API
-#pod references, technical articles and more.
-#pod
-#pod =item L<Business::Tax::Avalara|Business::Tax::Avalara>
-#pod
-#pod An alternative that uses Avalara's REST API.
-#pod
-#pod =item L<XML::Compile::SOAP|XML::Compile::SOAP> and L<XML::Compile::WSDL11|XML::Compile::WSDL11>
-#pod
-#pod Part of the L<XML::Compile|XML::Compile> suite
-#pod and the basis for this distribution. It's helpful to understand these in
-#pod order to debug or extend this module.
-#pod
-#pod =back
-#pod
 #pod =head1 METHODS
 #pod
 #pod Aside from the L</new> method, L</services> attribute and
@@ -127,8 +108,9 @@ sub _new_service {
 #pod containing the results of the SOAP call; in list context they return the
 #pod results hashref and an
 #pod L<XML::Compile::SOAP::Trace|XML::Compile::SOAP::Trace>
-#pod object suitable for debugging and exception handling. If there is no result
-#pod then an exception will be thrown.
+#pod object suitable for debugging and exception handling.
+#pod
+#pod If there is no result then you should check the trace object for why.
 #pod
 #pod Please consult the
 #pod L<Avalara SOAP API reference|http://developer.avalara.com/api-reference>
@@ -142,7 +124,8 @@ sub _new_service {
 #pod
 #pod SOAP operation names have been transformed from C<CamelCase> to
 #pod C<lowercase_with_underscores>. For example, C<GetTax> is now
-#pod L</get_tax>.
+#pod L</get_tax>. If you do not like this behavior then use
+#pod L<<< C<< orthodox => 1 >>|/orthodox >>> when calling L</new>.
 #pod
 #pod =item *
 #pod
@@ -166,6 +149,15 @@ sub _new_service {
 #pod
 #pod =back
 #pod
+#pod =attr orthodox
+#pod
+#pod When set to true at construction, the generated methods will exactly match
+#pod their C<CamelCase> SOAP operation names.
+#pod
+#pod =cut
+
+has orthodox => ( is => 'ro', isa => Bool, default => 0 );
+
 #pod =for Pod::Coverage BUILD
 #pod
 #pod =cut
@@ -181,11 +173,12 @@ sub BUILD {
             %soap_params,
         );
         for my $operation ( $service->wsdl->operations(%soap_params) ) {
-
-            # normalize operation name into a Perl method name
             my $method_name = $operation->name;
-            $method_name =~ s/ (?<= [[:alnum:]] ) ( [[:upper:]] ) /_\l$1/xmsg;
-            $method_name = lcfirst $method_name;
+            if ( not $self->orthodox ) {    # normalize operation name
+                $method_name
+                    =~ s/ (?<= [[:alnum:]] ) ( [[:upper:]] ) /_\l$1/xmsg;
+                $method_name = lcfirst $method_name;
+            }
 
             $self->_stash->add_symbol(
                 "&$method_name" => _method_closure( $service, $operation ) );
@@ -226,6 +219,10 @@ sub _method_closure {
         my $wsdl = $service->wsdl;
         $service->_current_operation_name( $operation->name );
 
+        if ( 'GetTax' eq $operation ) {
+            @parameters = _today_to_docdate(@parameters);
+        }
+
         my ( $answer_ref, $trace ) = $wsdl->call(
             $operation->name => {
                 Profile => {
@@ -241,16 +238,24 @@ sub _method_closure {
                 },
             },
         );
-
-        if ( not $answer_ref ) {
-            for ( $trace->errors ) { $_->throw }
+        if ( 'HASH' eq ref $answer_ref ) {
+            $answer_ref
+                = $answer_ref->{parameters}{ $operation->name . 'Result' };
         }
-        $answer_ref
-            = $answer_ref->{parameters}{ $operation->name . 'Result' };
         return wantarray ? ( $answer_ref, $trace ) : $answer_ref;
     };
 }
 
+sub _today_to_docdate {
+    my %parameters = @_;
+    if ( not defined $parameters{DocDate} ) {
+        $parameters{DocDate}
+            = DateTime::Format::XSD->format_datetime( DateTime->today );
+        $parameters{DocDate} =~ s/ T .* \z//xms;
+    }
+    return %parameters;
+}
+
 1;
 
 __END__
@@ -268,7 +273,7 @@ WebService::Avalara::AvaTax - Avalara SOAP interface as compiled Perl methods
 
 =head1 VERSION
 
-version 0.009
+version 0.017
 
 =head1 SYNOPSIS
 
@@ -301,8 +306,9 @@ necessary parameters. In scalar context they return a reference to a hash
 containing the results of the SOAP call; in list context they return the
 results hashref and an
 L<XML::Compile::SOAP::Trace|XML::Compile::SOAP::Trace>
-object suitable for debugging and exception handling. If there is no result
-then an exception will be thrown.
+object suitable for debugging and exception handling.
+
+If there is no result then you should check the trace object for why.
 
 Please consult the
 L<Avalara SOAP API reference|http://developer.avalara.com/api-reference>
@@ -316,7 +322,8 @@ and more Perl-ish, the following changes have been made:
 
 SOAP operation names have been transformed from C<CamelCase> to
 C<lowercase_with_underscores>. For example, C<GetTax> is now
-L</get_tax>.
+L</get_tax>. If you do not like this behavior then use
+L<<< C<< orthodox => 1 >>|/orthodox >>> when calling L</new>.
 
 =item *
 
@@ -349,6 +356,13 @@ that can be set at construction.
 
 =head2 get_tax
 
+I<< (SOAP operation: C<GetTax>) >>
+
+As a convenience to
+L<Business::Tax::Avalara|Business::Tax::Avalara>
+users (and others), the C<DocDate> element below will default to today's date
+in the UTC time zone.
+
 Constructing and making an example request:
 
     my %get_tax_request = (
@@ -393,14 +407,14 @@ Constructing and making an example request:
             TaxRegionId => 0,
         },
     );
-    for my $address_code (0 .. @addresses) {
+    for my $address_code (0 .. $#addresses) {
         push @{$get_tax_request{Addresses}{BaseAddress}} => {
             AddressCode => $address_code,
             %{ $addresses[$address_code] },
         };
     }
 
-    my @lines => (
+    my @lines = (
         {   OriginCode      => 0,
             DestinationCode => 1,
             ItemCode        => 'N543',
@@ -429,7 +443,7 @@ Constructing and making an example request:
             Description     => 'Shipping Charge',
         },
     );
-    for my $line_no (0 .. $#lines) {
+    for my $line_no (1 .. @lines) {
         push @{$get_tax_request{Lines}{Line}} => {
             No => $line_no,
             %{ $lines[$line_no - 1] },
@@ -440,6 +454,8 @@ Constructing and making an example request:
 
 =head2 post_tax
 
+I<< (SOAP operation: C<PostTax>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->post_tax(
@@ -455,6 +471,8 @@ Example:
 
 =head2 commit_tax
 
+I<< (SOAP operation: C<CommitTax>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->commit_tax(
@@ -466,6 +484,8 @@ Example:
 
 =head2 cancel_tax
 
+I<< (SOAP operation: C<CancelTax>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->cancel_tax(
@@ -477,6 +497,8 @@ Example:
 
 =head2 adjust_tax
 
+I<< (SOAP operation: C<AdjustTax>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->adjust_tax(
@@ -576,6 +598,8 @@ Example:
 
 =head2 get_tax_history
 
+I<< (SOAP operation: C<GetTaxHistory>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->get_tax_history(
@@ -587,6 +611,8 @@ Example:
 
 =head2 validate
 
+I<< (SOAP operation: C<Validate>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->validate(
@@ -605,6 +631,8 @@ Example:
 
 =head2 is_authorized
 
+I<< (SOAP operation: C<IsAuthorized>) >>
+
 Both
 L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
 and
@@ -617,6 +645,9 @@ method on its
 L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
 attribute.
 
+Note that the parameter passed to this call is a comma-delimited list of
+SOAP operation names in C<CamelCase>, not C<lowercase_with_underscores>.
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->is_authorized(
@@ -634,6 +665,8 @@ Example:
 
 =head2 ping
 
+I<< (SOAP operation: C<Ping>) >>
+
 Both
 L<WebService::Avalara::AvaTax::Service::Address|WebService::Avalara::AvaTax::Service::Address>
 and
@@ -646,6 +679,9 @@ method on its
 L<wsdl|WebService::Avalara::AvaTax::Role::Connection/wsdl>
 attribute.
 
+Note that this method does support a single string as a message parameter;
+this is effectively ignored though.
+
 Example:
 
     use List::Util 1.33 'any';
@@ -659,6 +695,8 @@ Example:
 
 =head2 tax_summary_fetch
 
+I<< (SOAP operation: C<TaxSummaryFetch>) >>
+
 Example:
 
     my ( $answer_ref, $trace ) = $avatax->tax_summary_fetch(
@@ -669,6 +707,8 @@ Example:
 
 =head2 apply_payment (DEPRECATED)
 
+I<< (SOAP operation: C<ApplyPayment>) >>
+
 From L<Avalara API documentation|http://developer.avalara.com/api-docs/soap/applypayment>:
 
 =over
@@ -693,6 +733,8 @@ Example:
 
 =head2 reconcile_tax_history (LEGACY API)
 
+I<< (SOAP operation: C<ReconcileTaxHistory>) >>
+
 From L<Avalara API documentation|http://developer.avalara.com/api-docs/soap/reconciletaxhistory>:
 
 =over
@@ -749,6 +791,13 @@ passed to both:
 
 =back
 
+=head2 orthodox
+
+When set to true at construction, the generated methods will exactly match
+their C<CamelCase> SOAP operation names.
+
+=for Pod::Coverage BUILD
+
 =head1 SEE ALSO
 
 =over
@@ -770,8 +819,6 @@ order to debug or extend this module.
 
 =back
 
-=for Pod::Coverage BUILD
-
 =head1 SUPPORT
 
 =head2 Perldoc
