@@ -4,7 +4,7 @@ package WebService::Avalara::AvaTax::Service::Tax;
 
 use Modern::Perl '2011';    ## no critic (Modules::ProhibitUseQuotedVersion)
 
-our $VERSION = '0.009';     # TRIAL VERSION
+our $VERSION = '0.017';     # TRIAL VERSION
 use utf8;
 
 #pod =head1 SYNOPSIS
@@ -27,10 +27,12 @@ use utf8;
 #pod
 #pod =cut
 
+use File::ShareDir::ProjectDistDir 'dist_file';
 use Moo;
 use URI;
 use XML::Compile::SOAP11;
 use XML::Compile::SOAP12;
+use XML::Compile::WSDL11;
 use namespace::clean;
 with 'WebService::Avalara::AvaTax::Role::Connection';
 
@@ -71,6 +73,28 @@ has '+port' => ( default => 'TaxSvcSoap' );
 
 has '+service' => ( default => 'TaxSvc' );
 
+#pod =head1 WORKAROUNDS FOR INCORRECT AVALARA RESPONSES
+#pod
+#pod As of this writing the Avalara SOAP API returns responses that are
+#pod inconsistent with the WSDL document provided. Specifically, the C<TaxIncluded>
+#pod and C<GeocodeType> fields in the C<GetTaxResponse> are wrongly placed in their
+#pod sequences of fields. This module adds an overlay to the Avalara tax service
+#pod WSDL file that attempts to work around these problems so that the responses
+#pod may be successfully parsed.
+#pod
+#pod =cut
+
+has '+wsdl' => (
+    default => sub {
+        my $self = shift;
+        my $wsdl = XML::Compile::WSDL11->new(
+            $self->user_agent->get( $self->uri )->content );
+        $wsdl->addWSDL(
+            dist_file( 'WebService-Avalara-AvaTax', 'taxsvc_patch.wsdl' ) );
+        return $wsdl;
+    },
+);
+
 1;
 
 __END__
@@ -88,7 +112,7 @@ WebService::Avalara::AvaTax::Service::Tax - Avalara AvaTax tax engine service vi
 
 =head1 VERSION
 
-version 0.009
+version 0.017
 
 =head1 SYNOPSIS
 
@@ -129,6 +153,15 @@ Defaults to C<AddressSvcSoap>.
 The SOAP service name used in the WSDL file at L</uri>.
 Defaults to C<AddressSvc>.
 
+=head1 WORKAROUNDS FOR INCORRECT AVALARA RESPONSES
+
+As of this writing the Avalara SOAP API returns responses that are
+inconsistent with the WSDL document provided. Specifically, the C<TaxIncluded>
+and C<GeocodeType> fields in the C<GetTaxResponse> are wrongly placed in their
+sequences of fields. This module adds an overlay to the Avalara tax service
+WSDL file that attempts to work around these problems so that the responses
+may be successfully parsed.
+
 =head1 SUPPORT
 
 =head2 Perldoc
